program BC;

// Bus-Controller Version 1.03 vom 01.12.2013

DEVICE = mega1284P;

{$I mega1284_Register.INC}

CONST

  // ProcClock = 16384000;       {Hertz}
  UBRR_VAL : WORD = 7;  // 128000  ok
//  UBRR_VAL : WORD = 3;  // 256000  ok

  XON  : Byte = $11;
  XOFF : Byte = $13;

  // Befehlscode fuer den Adress Extender
  kein_Code       : Byte = 00;
  LOAD_ADRESS     : Byte = 01;   // laden einer neuen Adresse
  INC_ADRESS      : Byte = 02;   // Inkrement der geladenen Adresse
  ADRESS_TRISTATE : Byte = 03;   // Adressbus auf Treestate setzen
  WAIT_Z80ToBuf   : Byte = 04;   // Wait Z80-Bus to AE-Buffer
  WAIT_BufToBC    : Byte = 05;   // Wait AE-Buffer to Bus-Controller


  // Befehlscode fuer Z80CALL
  BC_CONIN        : Byte = 01;
  BC_CONST        : Byte = 02;
  BC_CONOUT       : Byte = 03;
  BC_LIST         : Byte = 04;
  BC_COPY_BDOS    : Byte = 05;
  BC_COPY_CCP     : Byte = 06;
  BC_READ_SEC     : Byte = 07;
  BC_WRITE_SEC    : Byte = 08;
  BC_TRACE_Z80    : Byte = 09;
  BC_SET_TIME     : Byte = 10;


  CPM_spt : Byte = 72;

  RF_FuellByte : Byte = $E5;

  BB_Laenge       : Word = 900;  // 3*300
  BB_Laenge_DIV_3 : Word = 300;  // 3*300


{--------------------------------------------------------------}


VAR

{
PB0 = frei
PB1 = /AECMD   A -> H / AE Command / Work in Mode
PB2 = /BCWCLR  A -> H / BC Wait Clear (Aktiv bei L)
PB3 = BCWA     A -> L / BC Wait (Aktiv bei H)
PB4 = /Z80WA   E -> H / Z80 Wait(Aktiv bei L)
}

  Taste_E[PINB,0]   : bit;
  AECMD_A[PortB,1]  : bit;
  BCWCLR_A[PortB,2] : bit;
  BCWA_A  [PortB,3] : bit;
  Z80WA_E [PINB, 4] : bit;


{
PC0 = /WAIT    E -> X
PC1 = /MREQ    A -> H
PC2 = /IORQ    A -> H
PC3 = /RD      A -> H
PC4 = /WR      A -> H
PC5 = /RESET   A -> H
PC6 = /BUSRQ   A -> H
PC7 = /BUSAK   E <- X
}

  WAIT_E [PINC, 0] : bit;

  MREQ_A [PortC,1] : bit;
  MREQ_E [PINC, 1] : bit;
  MREQ_D [DDRC, 1] : bit;

  IORQ_A [PortC,2] : bit;
  IORQ_E [PINC, 2] : bit;
  IORQ_D [DDRC, 2] : bit;

  RD_A   [PortC,3] : bit;
  RD_E   [PINC, 3] : bit;
  RD_D   [DDRC, 3] : bit;

  WR_A   [PortC,4] : bit;
  WR_E   [PINC, 4] : bit;
  WR_D   [DDRC, 4] : bit;

  RESET_A[PortC,5] : bit;

  BUSRQ_A[PortC,6] : bit;

  BUSAK_E[PINC, 7] : bit;

{
PD0 = RXD0      E <-
PD1 = TXD0      A ->
PD2 = /AECS     A -> H
PD3 = /AERQ     E <- X
PD4 = /Z80CALL  E <- X
PD5 = /Z80AK    A -> H
PD6 = LED_GRUEN A -> L
PD7 = LED_ROT   A -> L
}

  AECS_A [PortD,2]  : bit;
  AERQ_E [PIND,3]   : bit;
  Z80CALL_E[PIND,4] : bit;
  Z80AK_A[PortD,5]  : bit;
  LED_GRUEN[PortD,6]: bit;
  LED_ROT[PortD,7]  : bit;


  Zeichen : Byte;

  loop_in, Trace_bereit : Boolean;

  ConsoleIn : Byte;


  Steuer_String : String[50];
  Anzeige_String : String[50];

  // Variablen fuer die DISK-Verwaltung
  SECTOR, TRACK_L, TRACK_H, DISKNO,
  DMAAD_L, DMAAD_H : Byte;

  // Sektor-Puffer
  Sektor_Puffer : Array [0..127] of Byte;
  Sektor_Puffer_Zeiger : Word;

  // RAM-Floppy
  RF_15, RF_08, RF_00 : Byte;

  // Date/Time
  Datum_gesetzt : Boolean;


  SCB_DATE_HI, SCB_DATE_LO : Byte;

  // COPY_ARRAY_TO_RAM
  DATEN_ARRAY : Adress;
  Laenge_Array : Word;

CONST


// *** BIOS-Speicherzellen ***
// F0C6
RAM_Z80_High   : Byte = $F0;

RAM_Z80CALL_MD : Byte = $C6;
RAM_Z80CALL_P1 : Byte = $C7;
RAM_Z80CALL_P2 : Byte = $C8;
RAM_Z80CALL_P3 : Byte = $C9;
RAM_Z80CALL_P4 : Byte = $CA;
RAM_Z80CALL_P5 : Byte = $CB;
RAM_Z80CALL_P6 : Byte = $CC;

// F03A
SECTOR_ADRESS_HI : Byte = $F0;
SECTOR_ADRESS_LO : Byte = $3A;

// *** Transferadressen BDOS/CCP ***
BDOS_ADRESS_HI   : Byte = $E2;
CCP_ADRESS_HI    : Byte = $DA;


// BIOS - Z80-Code
{$I CPD5_TXT.INC}

// BDOS/ZSDOS - Z80-Code
{$I BDOS_TXT.INC}

// CCP - Z80-Code
{$I CCP_TXT.INC}

// RAM-Floppy
{$I DISK.INC}

{--------------------------------------------------------------}
{ functions }


{$I SYSTEM.INC}



procedure InitPorts;
begin
  // PORT A = Daten
  DDRA:=  %00000000;


{
PB0 = Taste    E -> L 

PB1 = /AECMD   A -> H / AE Command / Work in Mode
PB2 = /BCWCLR  A -> H / BC Wait Clear (Aktiv bei L)
PB3 = BCWA     A -> L / BC Wait (Aktiv bei H)
PB4 = /Z80WA   E -> H / Z80 Wait(Aktiv bei L)

PB5 = MOSI 
PB6 = MISO 
PB7 = SCK 
}
  // PORT B Voreinstellungen
  PORTB:= %00000111;

  //       EEEEAAEE
  DDRB:=  %00001110;

  // PORT B Voreinstellungen
  PORTB:= %00000111;

{
PC0 = /WAIT    E -> X
PC1 = /MREQ    A -> H
PC2 = /IORQ    A -> H
PC3 = /RD      A -> H
PC4 = /WR      A -> H
PC5 = /RESET   A -> H
PC6 = /BUSRQ   A -> H
PC7 = /BUSAK   E <- X
}

  // PORT C Voreinstellungen (BUSRQ ist aktiv)
  PORTC:= %00111110;

  //       EAAAAAAE
  DDRC:=  %01111110;

  // PORT C Voreinstellungen (BUSRQ ist aktiv)
  PORTC:= %00111110;

{
PD0 = RXD0      E <-
PD1 = TXD0      A ->
PD2 = /AECS     A -> H
PD3 = /AERQ     E <- X
PD4 = /Z80CALL  E <- X
PD5 = /Z80AK    A -> H
PD6 = LED_GRUEN A -> L
PD7 = LED_ROT   A -> L
}
  // PORT D Voreinstellungen
  PORTD:= %00100100;

  //       AAAEEAAE
  DDRD:=  %11100110;

  // PORT D Voreinstellungen
  PORTD:= %00100100;

end InitPorts;



procedure Init_DATEN_ARRAY;
begin
  DATEN_ARRAY:=ArrToAdress(CPD5_ARR);
end Init_DATEN_ARRAY;







// ******************************
// ***   AE-Hilfsprogramme    ***
// ******************************





// ein Byte an AE senden
procedure AE_WRITE_BYTE(WR_Byte : byte);
begin

  // warten bis ATMEGA162 bereit ist
  while BitLow(AERQ_E) do
  endwhile;

  // /AECMD aktiv schalten
  ClearBit(AECMD_A);   

  // Datenbus auf Ausgang
  DDRA:= %11111111; // D0-D7 A

  // Daten auf Datenbus legen
  PORTA:=WR_Byte;

  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;

  // AECS_A aktiv schalten (Daten koennen uebernommen werden)
  ClearBit(AECS_A);

  // warten bis Daten uebernommen wurden
  while BitHigh(AERQ_E) do
  endwhile;

  // Betriebsart Programmieren abschalten
  SetBit(AECS_A);  // AECS_A inaktiv

  // Uebertragung abgeschlossen

  DDRA:= %00000000; // Datenbus auf Eingang
  
  SetBit(AECMD_A);    // AECMD_A inaktiv

end AE_WRITE_BYTE; 






procedure AE_WRITE_ADRESS(a16, a08, a00 : Byte);
begin

  AE_WRITE_BYTE(LOAD_ADRESS);

  // adr_00_07
  AE_WRITE_BYTE(a00);

  // adr_08_15
  AE_WRITE_BYTE(a08);

  // adr_16_18
  AE_WRITE_BYTE(a16);

end AE_WRITE_ADRESS;



// ******************************
// ***   RAM-Hilfsprogramme   ***
// ******************************






procedure RAM_loeschen;
var
  Adresse_00_15 : Word;
  Adresse_16_18 : Byte;
  b1 : Byte;
begin

  SetBit(LED_ROT);

  AE_WRITE_ADRESS(0,0,0);

  // Chips inaktiv schalten
  SetBit(RD_A);
  SetBit(WR_A);
  SetBit(MREQ_A);
  SetBit(AECS_A);
  SetBit(AECMD_A);

  Adresse_00_15:=0;
  Adresse_16_18:=0;

  loop_in := false;

  AE_WRITE_BYTE(INC_ADRESS);

  DDRA := %11111111; // Daten alles Ausgänge

  // Daten-Port mit festem Wert voreinstellen
  PortA := RF_FuellByte; // Data


  for Adresse_16_18:=0 to 15 do


    for Adresse_00_15:=0 to $8FFF do

      if loop_in then
        // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
        while BitHigh(AERQ_E) do
        endwhile;
      else
        loop_in := true;
      endif;

      // jetzt Adressdecoder inaktivieren
      SetBit(AECS_A);

      ClearBit(MREQ_A);
      ClearBit(WR_A);

      NOP; // 6 x 64us
      NOP;
      NOP;
      NOP;
      NOP;
      NOP;

      SetBit(MREQ_A);
      SetBit(WR_A);

      // naechste Adresse anlegen durch Adress-Decoder
      while BitLow(AERQ_E) do
      endwhile;

      ClearBit(AECS_A);  // AECS_A aktiv


    endfor; { for Adresse_00_15:=0 to $FFFF do }


  endfor; { for Adresse_16_18:=0 to 7 do }

  DDRA :=  %00000000; // alles Eingänge

  // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
  while BitHigh(AERQ_E) do
  endwhile;

  // jetzt AdressExtender inaktivieren
  SetBit(AECS_A);  // AECS_A inaktiv
  SetBit(WR_A);    // WE_A inaktiv
  SetBit(RD_A);    // OE_A inaktiv

  // warten bis Adress-Decoder bereit ist
  while BitLow(AERQ_E) do
  endwhile;

  ClearBit(LED_ROT);

end RAM_loeschen;










procedure COPY_ARRAY_TO_RAM(ADR_HI : Byte);
var
  READ_DATA : Byte;
  Adresse : Word;
begin

  SetBit(LED_ROT);

  AE_WRITE_ADRESS(0,ADR_HI,0);

  // Chips inaktiv schalten
  SetBit(RD_A);
  SetBit(WR_A);
  SetBit(MREQ_A);
  SetBit(AECS_A);
  SetBit(AECMD_A);

  Adresse:=0;

  loop_in := false;

  AE_WRITE_BYTE(INC_ADRESS);

  DDRA := %11111111; // Daten alles Ausgänge
   
  for Adresse:=0 to Laenge_Array do

    if loop_in then
      // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
      while BitHigh(AERQ_E) do
      endwhile;
    else
      loop_in := true;
    endif;

    // jetzt Adressdecoder inaktivieren
    SetBit(AECS_A);

    // Daten lesen von ARRAY
    READ_DATA:=ReadAdrArray(DATEN_ARRAY,Adresse);

    // Daten schreiben in RAM
    PortA := READ_DATA; // Data

    ClearBit(MREQ_A);
    ClearBit(WR_A); 

    NOP; // 6 x 64us
    NOP;  
    NOP; 
    NOP;  
    NOP; 
    NOP; 

    SetBit(MREQ_A);
    SetBit(WR_A);
 
    
    // naechste Adresse anlegen durch Adress-Decoder
    while BitLow(AERQ_E) do
    endwhile;

    ClearBit(AECS_A);  // AECS_A aktiv
   
  endfor; 

  DDRA :=  %00000000; // alles Eingänge


  // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
  while BitHigh(AERQ_E) do
  endwhile;

  // jetzt AdressExtender inaktivieren
  SetBit(AECS_A);  // AECS_A inaktiv
  SetBit(WR_A);    // WE_A inaktiv
  SetBit(RD_A);    // OE_A inaktiv

  // warten bis Adress-Decoder bereit ist
  while BitLow(AERQ_E) do
  endwhile;

  ClearBit(LED_ROT);

end COPY_ARRAY_TO_RAM;








procedure COPY_BDOS_TO_RAM;
var
  READ_DATA : Byte;
  Adresse : Word;
begin

  SetBit(LED_ROT);

  AE_WRITE_ADRESS(0,BDOS_ADRESS_HI,0);  // auf E200H laden

  // Chips inaktiv schalten
  SetBit(RD_A);
  SetBit(WR_A);
  SetBit(MREQ_A);
  SetBit(AECS_A);
  SetBit(AECMD_A);

  Adresse:=0;

  loop_in := false;

  AE_WRITE_BYTE(INC_ADRESS);

  DDRA := %11111111; // Daten alles Ausgänge

  for Adresse:=0 to Laenge_BDOS_Array do

    if loop_in then
      // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
      while BitHigh(AERQ_E) do
      endwhile;
    else
      loop_in := true;
    endif;

    // jetzt Adressdecoder inaktivieren
    SetBit(AECS_A);

    // Daten lesen von ARRAY
    READ_DATA:=BDOS_ARR[Adresse];


    // Daten schreiben in RAM
    PortA := READ_DATA; // Data

    ClearBit(MREQ_A);
    ClearBit(WR_A);

    NOP; // 6 x 64us
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;

    SetBit(MREQ_A);
    SetBit(WR_A);


    // naechste Adresse anlegen durch Adress-Decoder
    while BitLow(AERQ_E) do
    endwhile;

    ClearBit(AECS_A);  // AECS_A aktiv

  endfor;

  DDRA :=  %00000000; // alles Eingänge


  // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
  while BitHigh(AERQ_E) do
  endwhile;

  // jetzt AdressExtender inaktivieren
  SetBit(AECS_A);  // AECS_A inaktiv
  SetBit(WR_A);    // WE_A inaktiv
  SetBit(RD_A);    // OE_A inaktiv

  // warten bis Adress-Decoder bereit ist
  while BitLow(AERQ_E) do
  endwhile;

  ClearBit(LED_ROT);

end COPY_BDOS_TO_RAM;











procedure COPY_CCP_TO_RAM;
var
  READ_DATA : Byte;
  Adresse : Word;
begin

  SetBit(LED_ROT);

  AE_WRITE_ADRESS(0,CCP_ADRESS_HI,0);  // auf E200H laden

  // Chips inaktiv schalten
  SetBit(RD_A);
  SetBit(WR_A);
  SetBit(MREQ_A);
  SetBit(AECS_A);
  SetBit(AECMD_A);

  Adresse:=0;

  loop_in := false;

  AE_WRITE_BYTE(INC_ADRESS);

  DDRA := %11111111; // Daten alles Ausgänge

  for Adresse:=0 to Laenge_CCP_Array do

    if loop_in then
      // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
      while BitHigh(AERQ_E) do
      endwhile;
    else
      loop_in := true;
    endif;

    // jetzt Adressdecoder inaktivieren
    SetBit(AECS_A);

    // Daten lesen von ARRAY
    READ_DATA:=CCP_ARR[Adresse];


    // Daten schreiben in RAM
    PortA := READ_DATA; // Data

    ClearBit(MREQ_A);
    ClearBit(WR_A);

    NOP; // 6 x 64us
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;

    SetBit(MREQ_A);
    SetBit(WR_A);


    // naechste Adresse anlegen durch Adress-Decoder
    while BitLow(AERQ_E) do
    endwhile;

    ClearBit(AECS_A);  // AECS_A aktiv

  endfor;

  DDRA :=  %00000000; // alles Eingänge


  // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
  while BitHigh(AERQ_E) do
  endwhile;

  // jetzt AdressExtender inaktivieren
  SetBit(AECS_A);  // AECS_A inaktiv
  SetBit(WR_A);    // WE_A inaktiv
  SetBit(RD_A);    // OE_A inaktiv

  // warten bis Adress-Decoder bereit ist
  while BitLow(AERQ_E) do
  endwhile;

  ClearBit(LED_ROT);

end COPY_CCP_TO_RAM;











procedure COPY_DISK_TO_RAM;
var
  READ_DATA : Byte;
  Adresse : Word;
begin

  SetBit(LED_ROT);

  AE_WRITE_ADRESS(1,0,0);  // auf E200H laden

  // Chips inaktiv schalten
  SetBit(RD_A);
  SetBit(WR_A);
  SetBit(MREQ_A);
  SetBit(AECS_A);
  SetBit(AECMD_A);

  Adresse:=0;

  loop_in := false;

  AE_WRITE_BYTE(INC_ADRESS);

  DDRA := %11111111; // Daten alles Ausgänge

  for Adresse:=0 to Laenge_DISK_Array do

    if loop_in then
      // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
      while BitHigh(AERQ_E) do
      endwhile;
    else
      loop_in := true;
    endif;

    // jetzt Adressdecoder inaktivieren
    SetBit(AECS_A);

    // Daten lesen von ARRAY
    READ_DATA:=DISK_ARR[Adresse];


    // Daten schreiben in RAM
    PortA := READ_DATA; // Data

    ClearBit(MREQ_A);
    ClearBit(WR_A);

    NOP; // 6 x 64us
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;

    SetBit(MREQ_A);
    SetBit(WR_A);


    // naechste Adresse anlegen durch Adress-Decoder
    while BitLow(AERQ_E) do
    endwhile;

    ClearBit(AECS_A);  // AECS_A aktiv

  endfor;

  DDRA :=  %00000000; // alles Eingänge


  // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
  while BitHigh(AERQ_E) do
  endwhile;

  // jetzt AdressExtender inaktivieren
  SetBit(AECS_A);  // AECS_A inaktiv
  SetBit(WR_A);    // WE_A inaktiv
  SetBit(RD_A);    // OE_A inaktiv

  // warten bis Adress-Decoder bereit ist
  while BitLow(AERQ_E) do
  endwhile;

  ClearBit(LED_ROT);

end COPY_DISK_TO_RAM;
















procedure COPY_Sektor_Puffer_TO_RAM(SA15, SA08, SA00 : Byte);
var
  READ_DATA : Byte;
  Adresse : Word;
begin

  SetBit(LED_ROT);

  AE_WRITE_ADRESS(SA15,SA08,SA00);  // auf E200H laden

  // Chips inaktiv schalten
  SetBit(RD_A);
  SetBit(WR_A);
  SetBit(MREQ_A);
  SetBit(AECS_A);
  SetBit(AECMD_A);

  Adresse:=0;

  loop_in := false;

  AE_WRITE_BYTE(INC_ADRESS);

  DDRA := %11111111; // Daten alles Ausgänge

  for Adresse:=0 to 127 do

    if loop_in then
      // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
      while BitHigh(AERQ_E) do
      endwhile;
    else
      loop_in := true;
    endif;

    // jetzt Adressdecoder inaktivieren
    SetBit(AECS_A);

    // Daten lesen von ARRAY
    READ_DATA:=Sektor_Puffer[Adresse];


    // Daten schreiben in RAM
    PortA := READ_DATA; // Data

    ClearBit(MREQ_A);
    ClearBit(WR_A);

    NOP; // 6 x 64us
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;

    SetBit(MREQ_A);
    SetBit(WR_A);


    // naechste Adresse anlegen durch Adress-Decoder
    while BitLow(AERQ_E) do
    endwhile;

    ClearBit(AECS_A);  // AECS_A aktiv

  endfor;

  DDRA :=  %00000000; // alles Eingänge


  // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
  while BitHigh(AERQ_E) do
  endwhile;

  // jetzt AdressExtender inaktivieren
  SetBit(AECS_A);  // AECS_A inaktiv
  SetBit(WR_A);    // WE_A inaktiv
  SetBit(RD_A);    // OE_A inaktiv

  // warten bis Adress-Decoder bereit ist
  while BitLow(AERQ_E) do
  endwhile;

  ClearBit(LED_ROT);

end COPY_Sektor_Puffer_TO_RAM;










procedure COPY_RAM_TO_Sektor_Puffer(SA15, SA08, SA00 : Byte);
var
  READ_DATA : Byte;
  Adresse : Word;
begin

  SetBit(LED_ROT);

  AE_WRITE_ADRESS(SA15,SA08,SA00);

  // Chips inaktiv schalten
  SetBit(RD_A);
  SetBit(WR_A);
  SetBit(MREQ_A);
  SetBit(AECS_A);
  SetBit(AECMD_A);

  Adresse:=0;

  loop_in := false;

  AE_WRITE_BYTE(INC_ADRESS);

  DDRA :=  %00000000; // alles Eingänge

  for Adresse:=0 to 127 do

    if loop_in then
      // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
      while BitHigh(AERQ_E) do
      endwhile;
    else
      loop_in := true;
    endif;

    // jetzt Adressdecoder inaktivieren
    SetBit(AECS_A);


    ClearBit(MREQ_A);
    ClearBit(RD_A);

    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;

    READ_DATA:=PINA;

    SetBit(RD_A);
    SetBit(MREQ_A);


    // Daten schreiben in ARRAY
    Sektor_Puffer[Adresse]:=READ_DATA;


    // naechste Adresse anlegen durch Adress-Decoder
    while BitLow(AERQ_E) do
    endwhile;

    ClearBit(AECS_A);  // AECS_A aktiv

  endfor;


  // warten falls Adress-Decoder noch beschaeftigt ist (unwahrscheinlich)
  while BitHigh(AERQ_E) do
  endwhile;

  // jetzt AdressExtender inaktivieren
  SetBit(AECS_A);  // AECS_A inaktiv
  SetBit(WR_A);    // WE_A inaktiv
  SetBit(RD_A);    // OE_A inaktiv

  // warten bis Adress-Decoder bereit ist
  while BitLow(AERQ_E) do
  endwhile;

  ClearBit(LED_ROT);

end COPY_RAM_TO_Sektor_Puffer;










function lese_RAM_Zelle : Byte;
var
  RAM_Zelle : Byte;
begin

  DDRA :=  %00000000; // alles Eingänge

 // Chips inaktiv schalten
  SetBit(RD_A);
  SetBit(WR_A);
  SetBit(AECS_A);
  SetBit(AECMD_A);    // AECMD_A inaktiv

  // naechste Adresse anlegen durch Adress-Decoder
  while BitLow(AERQ_E) do
  endwhile;

  ClearBit(MREQ_A);
  ClearBit(RD_A);  

  NOP; 
  NOP;  
  NOP; 
  NOP;  
  NOP; 
  NOP;

  RAM_Zelle:=PINA;

  SetBit(RD_A);
  SetBit(MREQ_A);

  RESULT:=RAM_Zelle;

end lese_RAM_Zelle;




procedure INC_Adresse;
begin

  ClearBit(AECS_A);  // AECS_A aktiv

  // naechste Adresse anlegen durch Adress-Decoder
  while BitHigh(AERQ_E) do
  endwhile;

  SetBit(AECS_A);  // AECS_A inaktiv

  // naechste Adresse anlegen durch Adress-Decoder
  while BitLow(AERQ_E) do
  endwhile;

end INC_Adresse;




function lese_RAM_Zelle_INC : Byte;
var
  RAM_Zelle : Byte;
begin
  RAM_Zelle:=lese_RAM_Zelle;
  INC_Adresse;

  RESULT:=RAM_Zelle;

end lese_RAM_Zelle_INC;





procedure schreibe_RAM_Zelle(Wert_Zelle : Byte);
begin

  // Chips inaktiv schalten
  SetBit(RD_A);
  SetBit(WR_A);
  SetBit(MREQ_A);
  SetBit(AECS_A);
  SetBit(AECMD_A);

  DDRA :=  %11111111; // alles Ausgaenge

  // naechste Adresse anlegen durch Adress-Decoder
  while BitLow(AERQ_E) do
  endwhile;

  // Daten schreiben in RAM
  PortA := Wert_Zelle; // Data

  ClearBit(MREQ_A);
  ClearBit(WR_A);

  NOP; // 6 x 64us
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;

  SetBit(MREQ_A);
  SetBit(WR_A);

  DDRA :=  %00000000; // alles Eingänge

end schreibe_RAM_Zelle;












// ******************************
// ***   BUS-Hilfsprogramme   ***
// ******************************





procedure BC_Bus_anfordern;
begin

  // Busanforderung an Z80
  SetBit(LED_ROT);
  ClearBit(BUSRQ_A);

  while BitHigh(BUSAK_E) do
  endwhile;

  NOP; 
  NOP;
  NOP; 
  NOP;
 
  {
  PC0 = frei     E -> X
  PC1 = /MREQ    A -> H
  PC2 = /IORQ    A -> H
  PC3 = /RD      A -> H
  PC4 = /WR      A -> H
  PC5 = /RESET   A -> H
  PC6 = /BUSRQ   A -> H
  PC7 = /BUSAK   E <- X
  }

  // PORT C Voreinstellungen (BUSRQ ist aktiv)
  SetBit(MREQ_A);
  SetBit(IORQ_A);
  SetBit(RD_A);
  SetBit(WR_A);


  // PORT C Steuerleitungen als Ausgang schalten
  SetBit(MREQ_D);
  SetBit(IORQ_D);
  SetBit(RD_D);
  SetBit(WR_D);


  // PORT C Voreinstellungen (BUSRQ ist aktiv)
  SetBit(MREQ_A);
  SetBit(IORQ_A);
  SetBit(RD_A);
  SetBit(WR_A);

 

  ClearBit(LED_ROT);

end BC_Bus_anfordern;




procedure BC_Bus_freigeben;
begin

  SetBit(LED_ROT);

  // PORT A = Daten
  DDRA:=  %00000000;

  // PORT B bleibt so

  {
  PC0 = frei     E -> X
  PC1 = /MREQ    A -> H
  PC2 = /IORQ    A -> H
  PC3 = /RD      A -> H
  PC4 = /WR      A -> H
  PC5 = /RESET   A -> H
  PC6 = /BUSRQ   A -> H
  PC7 = /BUSAK   E <- X
  }
  // PORT C Voreinstellungen (BUSRQ ist aktiv), Reset bleibt Ausgang
  // alle anderen Z80-Steuerleitungen werden
  // auf Eingang gestellt und freigegeben

  ClearBit(MREQ_D);
  ClearBit(IORQ_D);
  ClearBit(RD_D);
  ClearBit(WR_D);

  // PORT D bleibt so


  ClearBit(MREQ_A);
  ClearBit(IORQ_A);
  ClearBit(RD_A);
  ClearBit(WR_A);

  NOP; 
  NOP;
  NOP; 
  NOP;
 
  // jetzt wird die Z80 frei gegeben
  SetBit(BUSRQ_A);

  // warten bis Z80 den Bus uebernommen hat
  while BitLow(BUSAK_E) do
  endwhile;

  ClearBit(LED_ROT);

end BC_Bus_freigeben;






procedure starte_Z80;
begin

  // der Adress Extender gibt die Adressleitungen frei
  AE_WRITE_BYTE(ADRESS_TRISTATE);

  // als Erstes ziehen wir uns schrittweise zurueck
  BC_Bus_freigeben;

  // zur Bestaetigung wird die gruene LED eingeschaltet
  SetBit(LED_GRUEN);

  // fuer einen definierten Zustand wird
  // die Z80 noch einmal zurück gesetzt

  ClearBit(RESET_A);
  NOP;
  NOP;		// 20 Takte BC sind bei 4 MHz etwa 7 Takte Z80
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  NOP;
  SetBit(RESET_A);


end starte_Z80;






procedure lese_WAIT_Buffer;
var
  Wert_Byte : Byte;
begin
  // warten bis ATMEGA162 bereit ist
  while BitLow(AERQ_E) do
  endwhile;

  // /AECMD inaktiv schalten
  SetBit(AECMD_A);

  // AECS_A aktiv schalten (Buszustand wird in AE gelesen)
  ClearBit(AECS_A);

  // warten bis AE fertig ist
  while BitHigh(AERQ_E) do
  endwhile;

  NOP;
  NOP;
  NOP;
  NOP;

  Wert_Byte := PinA;

  SetBit(AECS_A);  // AECS_A inaktiv

  ByteToString(Wert_Byte);

end lese_WAIT_Buffer;





procedure starte_WAIT;
var
  Wait_Zaehler : Word;
begin

  // Initialisieren des AE
  BC_Bus_anfordern;

  // der Adress Extender wird fuer das Einlesen initialisiert
  AE_WRITE_BYTE(WAIT_Z80ToBuf);

  BC_Bus_freigeben;

  // WAIT-Logik einschalten
  SetBit(BCWA_A);

  for Wait_Zaehler:=1 to 300 do

    // warten bis der M1-Zyklus der Z80 die Wait-Logik aktiviert
    // hat und /WAIT auf Low setzt
    while BitHigh(WAIT_E) do
    endwhile;

    wait_ms(1);

    // /AECMD inaktiv schalten
    SetBit(AECMD_A);

    // AECS_A aktiv schalten (Buszustand wird in AE gelesen)
    ClearBit(AECS_A);

    // warten bis AE fertig ist
    while BitHigh(AERQ_E) do
    endwhile;

    // AECS_A inaktiv schalten
    SetBit(AECS_A);  // AECS_A inaktiv

    // warten bis ATMEGA162 bereit ist
    while BitLow(AERQ_E) do
    endwhile;

   // Setzen der Wait-Logik
    ClearBit(BCWCLR_A);  // Low aktiv
    NOP;
    SetBit(BCWCLR_A);  // Low aktiv

  endfor; { for Wait_Zaehler:=1 to 100 do }


  // Abschalten der WAIT-Logik
  ClearBit(BCWA_A);

  BC_Bus_anfordern;

  // Initialisieren de AE
  AE_WRITE_BYTE(WAIT_BufToBC);

  // PORT A = Daten Eingang
  DDRA:=  %00000000;

  Write_String:='';

  for Wait_Zaehler:=1 to 300 do
    lese_WAIT_Buffer;
    Anzeige_String:='c'+Write_String;
    lese_WAIT_Buffer;
    Anzeige_String:=Anzeige_String+Write_String;
    lese_WAIT_Buffer;
    Write_String:=Anzeige_String+Write_String;
    WriteLn;
  endfor; { for Wait_Zaehler:=1 to 100 do }


  // der Adress Extender gibt die Adressleitungen frei
  AE_WRITE_BYTE(ADRESS_TRISTATE);

  BC_Bus_freigeben;


end starte_WAIT;






// ***********************************************
// ***** Kommunikationsprogramme RAMFLOPPY  *****
// ***********************************************







procedure Sektor_der_RAMFLOPPY_berechnen;
var
  RF_Adresse : DWord;
  w1 : Word;
  w2 : Word;
  b1 : Byte;
  b2 : Byte;
  b3 : Byte;
begin
                   {
  SECTOR:=lese_RAM_Zelle_INC;
  TRACK_L:=lese_RAM_Zelle_INC;
  TRACK_H:=lese_RAM_Zelle_INC;
  DISKNO:=lese_RAM_Zelle_INC;
  DMAAD_L:=lese_RAM_Zelle_INC;
  DMAAD_H:=lese_RAM_Zelle_INC;

  I_POS:=(VAR_WR_LFD_NR*16)+(((VAR_TRACK*72)+(VAR_SECTOR))*128);
               CPM_spt
  }
  b1:=CPM_spt;           // 72
  w1:=ByteToWord(b1);
  b1:=TRACK_L;           // TRACK
  w2:=ByteToWord(b1);
  w1:=w1*w2;             // TRACK*72
  b1:=SECTOR;
  w2:=ByteToWord(b1);    // SECTOR
  w1:=w1+w2;             // (TRACK*72)+SECTOR
  RF_Adresse:=WordToDW(w1);
  SHL(RF_Adresse);
  SHL(RF_Adresse);
  SHL(RF_Adresse);
  SHL(RF_Adresse);
  SHL(RF_Adresse);
  SHL(RF_Adresse);
  SHL(RF_Adresse);  // ((TRACK*72)+SECTOR)*128
  RF_Adresse:=RF_Adresse+$10000;

  RF_00:=DWByte(0,RF_Adresse);
  RF_08:=DWByte(1,RF_Adresse);
  RF_15:=DWByte(2,RF_Adresse);

end Sektor_der_RAMFLOPPY_berechnen;







// ***********************************************
// ***** Kommunikationsprogramme mit dem PC  *****
// ***********************************************





// liest die Bytes aus der SteuerString in den BSECTOR
procedure Steuer_String_To_Sector;
var
  b1 : Byte;
  b2 : Byte;
  b3 : Byte;
  c1 : Char;
begin
  b2:=2;
  for b1:=0 to 15 do
    c1:=Steuer_String[b2];
    Inc(b2);
    Write_String:=c1;
    c1:=Steuer_String[b2];
    Inc(b2);
    Write_String:=Write_String+c1;
    b3:=StringToByte;
    Sektor_Puffer[Sektor_Puffer_Zeiger]:=b3;
    Inc(Sektor_Puffer_Zeiger);
  endfor;

end Steuer_String_To_Sector;






// liest die Bytes aus der SteuerString in den BSECTOR
procedure Steuer_String_To_Sector_Komp;
var
 // x1 : Byte;
  x2 : Byte;
  DB7 : Byte;
  DB8 : Byte;
  BitPos_DB8 : Byte;
  BitPos_DB7 : Byte;
  c1 : Char;
  Pos_SS : Byte;
  ein_Bit : Boolean;

begin
  BitPos_DB8:=7;
  DB8:=$00;
  for Pos_SS:=2 to 38 do

    c1:=Steuer_String[Pos_SS];
    DB7:=CharToByte(c1);

    BitPos_DB7:=7;
    for x2:=0 to 6 do

      Dec(BitPos_DB7); // 6..0
      ein_Bit:=Read_Bit_in_Byte(DB7,BitPos_DB7);

      if ein_Bit then
        DB8:=Set_Bit_in_Byte(DB8,BitPos_DB8);
      else
        DB8:=Clear_Bit_in_Byte(DB8,BitPos_DB8);
      endif;

      if BitPos_DB8=0 then

        Sektor_Puffer[Sektor_Puffer_Zeiger]:=DB8;
        Inc(Sektor_Puffer_Zeiger);

        BitPos_DB8:=7;
        DB8:=$00;

      else
        Dec(BitPos_DB8);
      endif;

    endfor; { for x2:=0 to 6 do }

  endfor; { for x1:=1 to 18 do }

end Steuer_String_To_Sector_Komp;






// liest die Bytes aus der SteuerString in den DATE und TIME-Puffer
procedure Steuer_String_To_TIME;
var
  b1 : Byte;
  b2 : Byte;
  c1 : Char;
begin

  // Date Low-Teil
  c1:=Steuer_String[4];
  Write_String:=c1;
  c1:=Steuer_String[5];
  Write_String:=Write_String+c1;
  b1:=StringToByte;

  AE_WRITE_ADRESS(0,SCB_DATE_HI,SCB_DATE_LO);
  schreibe_RAM_Zelle(b1);
  INC(SCB_DATE_LO);


  // Date High-Teil
  c1:=Steuer_String[2];
  Write_String:=c1;
  c1:=Steuer_String[3];
  Write_String:=Write_String+c1;
  b1:=StringToByte;

  AE_WRITE_ADRESS(0,SCB_DATE_HI,SCB_DATE_LO);
  schreibe_RAM_Zelle(b1);
  INC(SCB_DATE_LO);


  // HOUR
  c1:=Steuer_String[6];
  b1:=CharToByte(c1);
  b1:=b1-$30;
  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);
  c1:=Steuer_String[7];
  b2:=CharToByte(c1);
  b2:=b2-$30;
  b1:=b1+b2;

  AE_WRITE_ADRESS(0,SCB_DATE_HI,SCB_DATE_LO);
  schreibe_RAM_Zelle(b1);
  INC(SCB_DATE_LO);


  // MIN
  c1:=Steuer_String[8];
  b1:=CharToByte(c1);
  b1:=b1-$30;
  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);
  c1:=Steuer_String[9];
  b2:=CharToByte(c1);
  b2:=b2-$30;
  b1:=b1+b2;

  AE_WRITE_ADRESS(0,SCB_DATE_HI,SCB_DATE_LO);
  schreibe_RAM_Zelle(b1);
  INC(SCB_DATE_LO);


  // SEC
  c1:=Steuer_String[10];
  b1:=CharToByte(c1);
  b1:=b1-$30;
  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);
  c1:=Steuer_String[11];
  b2:=CharToByte(c1);
  b2:=b2-$30;
  b1:=b1+b2;

  AE_WRITE_ADRESS(0,SCB_DATE_HI,SCB_DATE_LO);
  schreibe_RAM_Zelle(b1);
  INC(SCB_DATE_LO);

  Datum_gesetzt:=true;

end Steuer_String_To_TIME;













procedure Steuer_String_auswerten;
var
  Buchstabe : Char;
  Zeichen_1 : Char;
  Zeichen_2 : Char;
  Laenge : Byte;
begin
  Laenge:=Length(Steuer_String);

  if Laenge>0 then
    Buchstabe:=Steuer_String[1];
    case Buchstabe of

            // T5B 2+1
      'T' : if Laenge=3 then
              Zeichen_1:=Steuer_String[2];
              Zeichen_2:=Steuer_String[3];
              Write_String:=Zeichen_1+Zeichen_2;
              ConsoleIn:=StringToByte;
            else
              Write_String:='x01';
              WriteLn;
            endif;

      |

            // UD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF  16*2+1
      'U' : if Laenge=33 then // ein Sektor-Abschnitt
              Steuer_String_To_Sector;
            else
              Write_String:='x03';
              WriteLn;
            endif;

      |

          // RB01B02B03B04..B36 36+1
      'R' : if Laenge=38 then // ein Sektor-Abschnitt
              Steuer_String_To_Sector_Komp;
            else
              Write_String:='x07';
              WriteLn;
            endif;
      |


          // D329B133357 = D + Datum(4) + Zeit(6)
      'D' : if Laenge=11 then
              Steuer_String_To_TIME;
            else
              Write_String:='x09';
              WriteLn;
            endif;
      |


    else
      Write_String:='x02';
      WriteLn;
    endcase;

  endif; { if Laenge>0 then }

end Steuer_String_auswerten;








procedure Steuer_String_lesen;
var
  akt_Zeichen : Char;
begin
  if Zeichen<>$0D then // 0D wird ignoriert

    if Zeichen=$0A then  // bei 0A wird ausgerwertet
      Steuer_String_auswerten;
      Steuer_String:='';
    else
      akt_Zeichen:=ByteToChar(Zeichen);
      Steuer_String:= Steuer_String + akt_Zeichen; // sonst nur abgespeichert
    endif; { if akt_Zeichen<>char($0A) }

  endif; { if akt_Zeichen<>char($0D) then }

end Steuer_String_lesen;





// ************************************




procedure STD_lesen;
begin

  AE_WRITE_ADRESS(0,SECTOR_ADRESS_HI,SECTOR_ADRESS_LO);
  SECTOR:=lese_RAM_Zelle_INC;
  TRACK_L:=lese_RAM_Zelle_INC;
  TRACK_H:=lese_RAM_Zelle_INC;
  DISKNO:=lese_RAM_Zelle_INC;
  DMAAD_L:=lese_RAM_Zelle_INC;
  DMAAD_H:=lese_RAM_Zelle_INC;

end STD_lesen;







procedure STD_an_PC_senden;
begin

  // Sector, Track, Drive (STD) = s SE TH TL DR
  ByteToString(SECTOR);
  Anzeige_String:='s'+Write_String;
  ByteToString(TRACK_H);
  Anzeige_String:=Anzeige_String+Write_String;
  ByteToString(TRACK_L);
  Anzeige_String:=Anzeige_String+Write_String;
  ByteToString(DISKNO);
  Write_String:=Anzeige_String+Write_String;

  // Senden der STD
  WriteLn;

end STD_an_PC_senden;






procedure Sektor_von_PC_lesen;
var
  b1 : Byte;
  b2 : Byte;
  SPZ : Word;
  LeseSchleife : Word;
begin

  Sektor_Puffer_Zeiger:=0;
  SPZ:=0;

  for b1:=0 to 7 do
    ByteToString(b1);
    Write_String:='u'+Write_String;
    WriteLn;

    LeseSchleife:=60000;
    while LeseSchleife>5 do
      RX_Puffer_abfragen;

      if Zeichen>0 then
        Steuer_String_lesen;
        if SPZ=Sektor_Puffer_Zeiger then
          LeseSchleife:=60000;
        else
          LeseSchleife:=1;  // Ende
          SPZ:=Sektor_Puffer_Zeiger;
        endif;
      endif;

      Dec(LeseSchleife);
    endwhile;

    if LeseSchleife>0 then
      Write_String:='x04';
      WriteLn;
    endif;

  endfor;

end Sektor_von_PC_lesen;







procedure Sektor_von_PC_lesen_Komp;
var
  b1 : Byte;
  b2 : Byte;
  SPZ : Word;
  LeseSchleife : Word;
begin

  Sektor_Puffer_Zeiger:=0;
  SPZ:=0;

  for b1:=0 to 3 do
    ByteToString(b1);
    Write_String:='r'+Write_String;
    WriteLn;

    LeseSchleife:=60000;
    while LeseSchleife>5 do
      RX_Puffer_abfragen;

      if Zeichen>0 then
        Steuer_String_lesen;
        if SPZ=Sektor_Puffer_Zeiger then
          LeseSchleife:=60000;
        else
          LeseSchleife:=1;  // Ende
          SPZ:=Sektor_Puffer_Zeiger;
        endif;
      endif;

      Dec(LeseSchleife);
    endwhile;

    if LeseSchleife>0 then
      Write_String:='x08';
      WriteLn;
    endif;

  endfor;

end Sektor_von_PC_lesen_Komp;







procedure Sektor_an_PC_senden;
var
  b1 : Byte;
  b2 : Byte;
  b3 : Byte;
begin

  Sektor_Puffer_Zeiger:=0;

  for b1:=0 to 7 do

    ByteToString(b1);
    Anzeige_String:='v'+Write_String;

    for b2:=0 to 15 do
      b3:=Sektor_Puffer[Sektor_Puffer_Zeiger];
      Inc(Sektor_Puffer_Zeiger);
      ByteToString(b3);
      Anzeige_String:=Anzeige_String+Write_String;
    endfor; { for b2:=0 to 15 do }

    Write_String:=Anzeige_String;
    WriteLn;

  endfor; { for b1:=0 to 7 do }

end Sektor_an_PC_senden;





procedure UP_READ_SEC;
begin

  STD_lesen;

  case DISKNO of

  0 : Sektor_der_RAMFLOPPY_berechnen;
      COPY_RAM_TO_Sektor_Puffer(RF_15,RF_08,RF_00);
      COPY_Sektor_Puffer_TO_RAM(0,DMAAD_H,DMAAD_L);
  |

  1 : STD_an_PC_senden;
      Sektor_von_PC_lesen_Komp;
      COPY_Sektor_Puffer_TO_RAM(0,DMAAD_H,DMAAD_L);
  |

  else
    Write_String:='x05';
    WriteLn;
  endcase;

end UP_READ_SEC;





procedure UP_WRITE_SEC;
begin

  STD_lesen;

  case DISKNO of

  0 : Sektor_der_RAMFLOPPY_berechnen;
      COPY_RAM_TO_Sektor_Puffer(0,DMAAD_H,DMAAD_L);
      COPY_Sektor_Puffer_TO_RAM(RF_15,RF_08,RF_00);

  |

  1 : STD_an_PC_senden;
      COPY_RAM_TO_Sektor_Puffer(0,DMAAD_H,DMAAD_L);
      Sektor_an_PC_senden;
  |

  else
    Write_String:='x06';
    WriteLn;
  endcase;

end UP_WRITE_SEC;









procedure UP_SET_TIME;
var
  LeseSchleife : Word;
begin

  Write_String:='d';
  WriteLn;

  Datum_gesetzt:=false;

  LeseSchleife:=60000;

  while LeseSchleife>5 do
    RX_Puffer_abfragen;

    if Zeichen>0 then
      Steuer_String_lesen;

      if Datum_gesetzt then
        LeseSchleife:=2;  // Ende
      else
        LeseSchleife:=60000;
      endif; { if Datum_gesetzt then }

    endif; { if Zeichen>0 then }

    Dec(LeseSchleife);
  endwhile;

  if LeseSchleife>2 then
    Write_String:='x10';
    WriteLn;
  endif;


end UP_SET_TIME;









procedure Z80CALL_abfragen;
var
  Mode : Byte;
  RAM_Zelle : Byte;
begin

  if BitLow(Z80CALL_E) then

    ClearBit(LED_GRUEN);

    BC_Bus_anfordern;

    AE_WRITE_ADRESS(0,RAM_Z80_High,RAM_Z80CALL_MD);
    Mode:=lese_RAM_Zelle;

    case Mode of

      BC_CONST  : AE_WRITE_ADRESS(0,RAM_Z80_High,RAM_Z80CALL_P1);
                  schreibe_RAM_Zelle(ConsoleIn);

      |

      BC_CONIN  : AE_WRITE_ADRESS(0,RAM_Z80_High,RAM_Z80CALL_P1);
                  schreibe_RAM_Zelle(ConsoleIn);
                  ConsoleIn:=0;

      |

      BC_CONOUT : AE_WRITE_ADRESS(0,RAM_Z80_High,RAM_Z80CALL_P1);
                  RAM_Zelle:=lese_RAM_Zelle;
                  ByteToString(RAM_Zelle);
                  Write_String:='o'+Write_String;
                  WriteLn;

      |

      BC_LIST   : AE_WRITE_ADRESS(0,RAM_Z80_High,RAM_Z80CALL_P1);
                  RAM_Zelle:=lese_RAM_Zelle;
                  ByteToString(RAM_Zelle);
                  Write_String:='l'+Write_String;
                  WriteLn;

      |

      BC_COPY_BDOS : COPY_BDOS_TO_RAM;

      |

      BC_COPY_CCP : COPY_CCP_TO_RAM;

      |

      BC_READ_SEC : UP_READ_SEC;

      |

      BC_WRITE_SEC : UP_WRITE_SEC;

      |

      BC_SET_TIME  : AE_WRITE_ADRESS(0,RAM_Z80_High,RAM_Z80CALL_P1);
                     SCB_DATE_LO:=lese_RAM_Zelle;
                     AE_WRITE_ADRESS(0,RAM_Z80_High,RAM_Z80CALL_P2);
                     SCB_DATE_HI:=lese_RAM_Zelle;
                     UP_SET_TIME;

      |

   else
     SetBit(LED_ROT);
   endcase;


    // der Adress Extender gibt die Adressleitungen frei
    AE_WRITE_BYTE(ADRESS_TRISTATE);

    BC_Bus_freigeben;

    ClearBit(Z80AK_A);

    // warten bis Z80 fertig ist
    while BitLow(Z80CALL_E) do
    endwhile;
    
    SetBit(Z80AK_A);

    SetBit(LED_GRUEN);

  endif;


end Z80CALL_abfragen;




function RF_leer : Boolean;
var
  RF_ist_leer : Boolean;
  Wert_Zelle : Byte;
begin
  RF_ist_leer:=true;
  AE_WRITE_ADRESS($FF,$FF,$FE);
  Wert_Zelle:=lese_RAM_Zelle_INC;
  if Wert_Zelle=RF_FuellByte then
    Wert_Zelle:=lese_RAM_Zelle_INC;
    if Wert_Zelle=RF_FuellByte then
      RF_ist_leer:=false;
    endif;
  endif;
  RESULT:=RF_ist_leer;
end RF_leer;






{--------------------------------------------------------------}
{ Main Program }
{$IDATA}







begin

ASM;

.equ UBRR0 = UBRR0L

ENDASM;

  // Grundeinstellung der Ports
  InitPorts;

  Init_DATEN_ARRAY;

  INIT_SER_PORT;

  ConsoleIn:=0;

  Trace_bereit:=true;

  ClearBit(LED_GRUEN);  // LED_GRUEN aus
  ClearBit(LED_ROT);  // LED_ROT aus


  // Abschalten der WAIT-Logik
  ClearBit(BCWA_A);  // High aktiv
  SetBit(BCWCLR_A);  // Low aktiv


  wait_ms(10);

  if RF_leer then

    // alle RAM-Zellen mit $FF vorbelegen
    RAM_loeschen;

    COPY_DISK_TO_RAM;

  endif;

  wait_ms(10);

  // BIOS kopieren
  Laenge_Array:=Laenge_CPD5_Array;
  DATEN_ARRAY:=ArrToAdress(CPD5_ARR);
  COPY_ARRAY_TO_RAM($00);

  wait_ms(10);

  sende_Zeichen(64);  // @  = Ruecksetzen der Hardware

  wait_ms(10);

  starte_Z80;

  loop

    RX_Puffer_abfragen;

    if Zeichen>0 then
      Steuer_String_lesen;
    endif;

    Z80CALL_abfragen;

    if BitLow(Z80WA_E) then
      if Trace_bereit then
        starte_WAIT;
        Trace_bereit:=false;
      endif;
    else
      Trace_bereit:=true;
    endif;

  endloop;


end BC.


